/* $Id: checksum.h,v 1.19 1996/08/29 09:48:07 davem Exp $ */
#ifndef __SPARC_CHECKSUM_H
#define __SPARC_CHECKSUM_H

/*  checksum.h:  IP/UDP/TCP checksum routines on the Sparc.
 *
 *  Copyright(C) 1995 Linus Torvalds
 *  Copyright(C) 1995 Miguel de Icaza
 *  Copyright(C) 1996 David S. Miller
 *  Copyright(C) 1996 Eddie C. Dost
 *
 * derived from:
 *	Alpha checksum c-code
 *      ix86 inline assembly
 *      RFC1071 Computing the Internet Checksum
 */

/*
 * computes the checksum of a memory block at buff, length len,
 * and adds in "sum" (32-bit)
 *
 * returns a 32-bit number suitable for feeding into itself
 * or csum_tcpudp_magic
 *
 * this function must be called with even lengths, except
 * for the last fragment, which may be odd
 *
 * it's best to have buff aligned on a 32-bit boundary
 */
extern unsigned int csum_partial(unsigned char * buff, int len, unsigned int sum);

/*
 * the same as csum_partial, but copies from fs:src while it
 * checksums
 *
 * here even more important to align src and dst on a 32-bit (or even
 * better 64-bit) boundary
 */
extern unsigned int csum_partial_copy(char *src, char *dst, int len, int sum);

#define csum_partial_copy_fromuser(s, d, l, w)  \
                       csum_partial_copy((char *) (s), (d), (l), (w))

/* ihl is always 5 or greater, almost always is 5, and iph is not always
 * word aligned, although that case is seldom.
 */
extern __inline__ unsigned short ip_fast_csum(__const__ unsigned char *iph,
					      unsigned int ihl)
{
	unsigned long tmp1, tmp2, tmp3, tmp4;
	unsigned short sum;

	__asm__ __volatile__("
		andcc	%1, 3, %%g0
		be,a	7f		! 4 byte aligned
		 sub	%2, 4, %2

		sll	%2, 2, %5	! number of bytes
		andcc	%1, 1, %%g0
		be,a	2f		! 2 byte aligned
		 mov	%%g0, %0

		ldub	[%1], %0
		dec	%5
		inc	%1

	2:
		andcc	%1, 2, %%g0
		be	3f		! 4 byte aligned
		 srl	%5, 1, %6	! number of half words

		lduh	[%1], %3
		add	%3, %0, %0	! no carry, add two half words
		add	%1, 2, %1
		sub	%5, 2, %5
		dec	%6

	3:
		srl	%6, 1, %6	! number of words
		ld	[%1], %4

	4:
		addcc	%4, %0, %0	! sum up words
		add	%1, 4, %1
		addx	%0, %%g0, %0
		subcc	%6, 1, %6
		be,a	5f
		 andcc	%5, 2, %%g0

		b	4b
		 ld	[%1], %4

	5:
		be,a	6f		! no half words left
		 andcc	%5, 1, %%g0

		lduh	[%1], %3
		addcc	%3, %0, %0
		add	%1, 2, %1
		addx	%0, %%g0, %0
		andcc	%5, 1, %%g0

	6:
		be,a	9f		! no bytes left
		 sll	%0, 16, %3

		ldub	[%1], %3	! single byte left
		sll	%3, 8, %3
		addcc	%3, %0, %0
		addx	%0, %%g0, %0
		sll	%0, 16, %3

		addcc	%0, %3, %3	! merge to half word
		srl	%3, 16, %0
		addx	%0, %%g0, %0

		srl	%0, 8, %3	! must have been odd if we get here
		and	%3, 0xff, %3
		sll	%0, 8, %0
		b	10f
		 add	%3, %0, %0

	7:
		ld	[%1 + 0x00], %0
		ld	[%1 + 0x04], %3
		addcc	%3, %0, %0
		ld	[%1 + 0x08], %4
		addxcc	%4, %0, %0
		ld	[%1 + 0x0c], %3
		addxcc	%3, %0, %0
		ld	[%1 + 0x10], %4
		addx	%0, %%g0, %0
	8:
		addcc	%4, %0, %0
		add	%1, 4, %1
		addxcc	%0, %%g0, %0
		subcc	%2, 1, %2
		be,a	9f
		 sll	%0, 16, %3

		b	8b
		 ld	[%1 + 0x10], %4
	9:
		addcc	%0, %3, %3
		srl	%3, 16, %0
		addx	%0, %%g0, %0
	10:
		xnor	%%g0, %0, %0
	" : "=r" (sum), "=&r" (iph), "=&r" (ihl),
	    "=r" (tmp1), "=r" (tmp2), "=r" (tmp3), "=r" (tmp4)
	  : "1" (iph), "2" (ihl));

	return sum;
}

/*
 * computes the checksum of the TCP/UDP pseudo-header
 * returns a 16-bit checksum, already complemented
 */
extern __inline__ unsigned short csum_tcpudp_magic(unsigned long saddr,
						   unsigned long daddr,
						   unsigned int len,
						   unsigned short proto,
						   unsigned int sum)
{
	__asm__ __volatile__("
		addcc	%1, %0, %0
		addxcc	%2, %0, %0
		addxcc	%3, %0, %0
		addx	%0, %%g0, %0
		sll	%0, 16, %1
		addcc	%1, %0, %0
		srl	%0, 16, %0
		addx	%0, %%g0, %0
		xnor	%%g0, %0, %0
	" : "=r" (sum), "=r" (saddr)
	  : "r" (daddr), "r" ((proto<<16)+len), "0" (sum), "1" (saddr));

	return sum;
}

/*
 *	Fold a partial checksum without adding pseudo headers
 */
extern __inline__ unsigned int csum_fold(unsigned int sum)
{
	unsigned int tmp;

	__asm__ __volatile__("
		addcc	%0, %1, %1
		srl	%1, 16, %1
		addx	%1, %%g0, %1
		xnor	%%g0, %1, %0
	" : "=&r" (sum), "=r" (tmp)
	  : "0" (sum), "1" (sum<<16));

	return sum;
}

/*
 * this routine is used for miscellaneous IP-like checksums, mainly
 * in icmp.c
 */
extern __inline__ unsigned short ip_compute_csum(unsigned char * buff, int len)
{
	return csum_fold(csum_partial(buff, len, 0));
}

#endif /* !(__SPARC_CHECKSUM_H) */
